<?php
# Duck Typing in PHP

# 1. Schnittstellen sind eine ausgezeichnete Möglichkeit, Wiederverwendbarkeit
# zu implementieren. Sie können eine allgemeine Schnittstelle für eine Reihe
# von Situationen erstellen (z.B. Sichern/Laden von der Plattenschnittstelle.)
# Sie können dann die Schnittstelle auf unterschiedliche Weise implementieren
# (z.B. für Formate wie tabulatorgetrenntes ASCII, XML und eine Datenbank). Sie
# können Code schreiben, der das Objekt auffordert, "sich selbst auf Disk zu
# speichern", ohne sich darum kümmern zu müssen, was das für das in Frage
# stehende Objekt bedeutet. Ein Objekt könnte sich selbst in der Datenbank
# speichern, ein anderes in ein XML und Sie können dieses Verhalten ändern
# über Zeit, ohne den Aufrufcode neu schreiben zu müssen.
# 
# Dies ermöglicht Ihnen, wiederverwendbaren Aufrufcode für beliebig viele
# verschiedene Objekte zu schreiben - Sie brauchen nicht zu wissen, was für ein
# Objekt es ist, solange es der allgemeinen Schnittstelle folgt.
# 
# 2. Schnittstellen können auch die kontinuierliche Entwicklung fördern.
interface KannFliegen {
  public function flieg();
}

interface KannSchwimmen {
  public function schwimm();
}

# abstract class: Klassen-Blaupause, kleinster gemeinsamer Nenner
abstract class Vogel {
  public function info() {
    echo "<p>Ihr Name ist {$this->name}.</p>";
    echo "<p>Sie ist ein Vogel.</p>";
  }
}

# Voegel Implementationen
class Taube extends Vogel implements KannFliegen {
  var $name = "Taube";

  public function flieg() {
    echo "<p>Sie kann fliegen.</p>";
  } 
}

class Pinguinöse extends Vogel implements KannSchwimmen {
  var $name = "Pinguinöse";

  public function schwimm() {
    echo "<p>Sie kann schwimmen.</p>";
  } 
}

class Ente extends Vogel implements KannFliegen, KannSchwimmen {
  var $name = "Ente";

  public function flieg() {
    echo "<p>Sie kann fliegen.</p>";
  }

  public function schwimm() {
    echo "<p>Sie kann schwimmen.</p>";
  }
}

# Aussenseiter
class Pferd {
	static $name = "Pferd";
}

# einfache Funktion, einen Vogel zu beobachten
function Ornitologe($vogel) {
  if ($vogel instanceof Vogel) {
    $vogel->info();
    if ($vogel instanceof KannFliegen) {
      $vogel->flieg();
    }
    if ($vogel instanceof KannSchwimmen) {
      $vogel->schwimm();
    }
  } else {
    echo "<p>{$vogel::$name} ist doch kein Vogel. So etwas beobachte ich nicht.</p>";
  }
}

# beobachte mir diese Voegel
Ornitologe(new Pinguinöse);

echo "<hr>";

Ornitologe(new Taube);

echo "<hr>";

Ornitologe(new Ente);

echo "<hr>";

Ornitologe(new Pferd);
